import { DeanApproval } from "@/data/deanApproval";
import { InternshipProposalAction, InternshipProposalActions } from "@/state/actions";
import { Internship } from "@/data";

export type ProposalStatus = "draft" | "awaiting" | "accepted" | "declined";

export type InternshipProposalState = {
    accepted: boolean;
    sent: boolean;
    declined: boolean;
    requiredDeanApprovals: DeanApproval[];
    proposal: Internship | null;
    comment: string | null;
}

const defaultInternshipProposalState: InternshipProposalState = {
    accepted: false,
    declined: false,
    proposal: null,
    requiredDeanApprovals: [],
    sent: false,
    comment: null
}

const getInternshipProposalStatus = ({ accepted, declined, sent }: InternshipProposalState): ProposalStatus => {
    switch (true) {
        case !sent:
            return "draft";
        case sent && accepted:
            return "accepted"
        case sent && declined:
            return "declined"
        case sent && (!accepted && !declined):
            return "awaiting"
        default:
            throw new Error("Invalid proposal state " + JSON.stringify({ accepted, declined, sent }));
    }
}

const internshipProposalReducer = (state: InternshipProposalState = defaultInternshipProposalState, action: InternshipProposalAction): InternshipProposalState => {
    switch (action.type) {
        case InternshipProposalActions.Save:
        case InternshipProposalActions.Send:
            return {
                ...state,
                proposal: action.internship
            }
        default:
            return state;
    }
}

export default internshipProposalReducer;
